@@ -1,11 +1,12 @@
 package Mail::Milter::Authentication::Metric;
 use strict;
 use warnings;
-our $VERSION = '2.20180612'; # VERSION
+our $VERSION = '2.20180726'; # VERSION
 
 
 use English qw{ -no_match_vars };
 use JSON;
+use TOML;
 use Mail::Milter::Authentication::Config qw{ get_config };
 use Mail::Milter::Authentication::Metric::Grafana;
 use Mail::Milter::Authentication::HTDocs;
@@ -333,7 +334,7 @@ sub child_handler {
 
     print $socket qq{</table>
 
-    <h2>Connection Details</h2>
+    <h2>Connection/Config Details</h2>
     <ul>};
     print $socket '<li>Protocol: ' . $config->{'protocol'} . '</li>';
     my $connections = $config->{'connections'};
@@ -342,6 +343,7 @@ sub child_handler {
         print $socket '<li>' . $connection . ': ' . $connections->{ $connection }->{ 'connection' } . '</li>'
     }
     print $socket qq{
+        <li><a href="/config">Effective config</a></li>
     </ul>
 
     <h2>Metrics</h2>
@@ -356,6 +358,14 @@ sub child_handler {
 </body>
 };
         }
+        elsif ( $request_uri eq '/config' ) {
+            print $socket "HTTP/1.0 200 OK\n";
+            print $socket "Content-Type: text/plain\n";
+            print $socket "\n";
+            my $toml = TOML::to_toml( $config );
+            $toml =~ s/\n\[/\n\n\[/g;
+            print $socket $toml;
+        }
         elsif ( $request_uri eq '/grafana' ) {
             print $socket "HTTP/1.0 200 OK\n";
             print $socket "Content-Type: application/json\n";
@@ -398,7 +408,7 @@ Mail::Milter::Authentication::Metric
 
 =head1 VERSION
 
-version 2.20180612
+version 2.20180726
 
 =head1 DESCRIPTION
 
