@@ -1,10 +1,11 @@
 package Mail::Milter::Authentication::Protocol::SMTP;
 use strict;
 use warnings;
-our $VERSION = '2.20180612'; # VERSION
+our $VERSION = '2.20180726'; # VERSION
 
 use English qw{ -no_match_vars };
 use Email::Date::Format qw{ email_date };
+use File::Temp;
 use IO::Socket;
 use IO::Socket::INET;
 use IO::Socket::UNIX;
@@ -596,7 +597,20 @@ sub smtp_command_data {
         }
     }
 
+    my $smtp_conf = $self->get_smtp_config();
+
+    my $chunk_limit = 1048576; # Process in chunks no larger than...
+    if ( exists ( $smtp_conf->{ 'chunk_limit' } ) ) {
+        $chunk_limit = $smtp_conf->{ 'chunk_limit' };
+    }
+
+    my $temp_file;
+    if ( exists ( $smtp_conf->{ 'temp_dir' } ) ) {
+        $temp_file = File::Temp->new( DIR => $smtp_conf->{ 'temp_dir' } );
+    }
+
     $self->smtp_status('smtp.i.body');
+    my $body_chunk;
     if ( ! $done ) {
         eval {
             alarm( $smtp->{'smtp_timeout_in'} );
@@ -609,14 +623,32 @@ sub smtp_command_data {
                 if ( $dataline =~ /^\./ ) {
                     $dataline = substr( $dataline, 1 );
                 }
-                $body .= $dataline;
+
+                if ( $temp_file ) {
+                    print $temp_file $dataline;
+                }
+                else {
+                   $body .= $dataline;
+                }
+
+                if ( length( $body_chunk ) + length( $dataline ) > $chunk_limit ) {
+                    $returncode = $handler->top_body_callback( $body_chunk );
+                    if ( $returncode != SMFIS_CONTINUE ) {
+                        $fail = 1;
+                    }
+                    $body_chunk = q{};
+                }
+
+                $body_chunk .= $dataline;
+
                 alarm( $smtp->{'smtp_timeout_in'} );
             }
             if ( ! $fail ) {
-                $returncode = $handler->top_body_callback( $body );
+                $returncode = $handler->top_body_callback( $body_chunk );
                 if ( $returncode != SMFIS_CONTINUE ) {
                     $fail = 1;
                 }
+                $body_chunk = q{};
             }
         };
         if ( my $error = $@ ) {
@@ -637,11 +669,23 @@ sub smtp_command_data {
     $self->smtp_status('smtp.i.data.received');
 
     if ( ! $fail ) {
-        $smtp->{'body'} = $body;
+
+        if ( $temp_file ) {
+            $smtp->{'spool'} = $temp_file;
+        }
+        else {
+            $smtp->{'body'} = $body;
+        }
 
         if ( $self->smtp_forward_to_destination() ) {
 
-            $handler->metric_count( 'mail_processed_total', { 'result' => 'accepted' } );
+            if ( my $reject_reason = $handler->get_quarantine_mail() ) {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'quarantined' } );
+            }
+            else {
+                $handler->metric_count( 'mail_processed_total', { 'result' => 'accepted' } );
+            }
+
             $handler->dbgout( 'Accept string', $smtp->{'string'}, LOG_INFO );
             $smtp->{'has_data'} = 1;
 
@@ -876,14 +920,32 @@ sub smtp_forward_to_destination {
     }
     $email .= "\r\n";
 
-    my $body = $smtp->{'body'};
-    $body =~ s/\015?\012/\015\012/g;
-    $email .= $body;
+    my $spool = $smtp->{'spool'};
+    if ( $spool ) {
 
-    # Handle transparency
-    $email =~ s/\015\012\./\015\012\.\./g;
+        # Handle transparency - should not be any in headers, but for completeness
+        $email =~ s/\015\012\./\015\012\.\./g;
 
-    print $sock $email;
+        print $sock $email;
+
+        seek( $spool, 0, 0 );
+        while ( my $line = <$spool> ) {
+            $line =~ s/\015?\012/\015\012/g;
+            $line =~ s/^\./\.\./g;
+            print $sock $line;
+        }
+
+    }
+    else {
+        my $body = $smtp->{'body'};
+        $body =~ s/\015?\012/\015\012/g;
+        $email .= $body;
+
+        # Handle transparency
+        $email =~ s/\015\012\./\015\012\.\./g;
+
+        print $sock $email;
+    }
 
     $self->logdebug( 'Sending end to destination' );
     $self->send_smtp_packet( $sock, '.',    '250' ) || return;
@@ -1014,7 +1076,7 @@ Mail::Milter::Authentication::Protocol::SMTP
 
 =head1 VERSION
 
-version 2.20180612
+version 2.20180726
 
 =head1 SYNOPSIS
 
